from lxml.etree import _Element
from openpyxl.styles.colors import ColorDescriptor, Color
from typing import (
    Iterator,
    List,
    Optional,
    Tuple,
    Union,
)
from openpyxl.descriptors import (
    Float,
    Set,
    Alias,
    NoneSet,
    Sequence,
    Integer,
    MinMax,
)

FILL_NONE = "none"
FILL_SOLID = "solid"
FILL_PATTERN_DARKDOWN = "darkDown"
FILL_PATTERN_DARKGRAY = "darkGray"
FILL_PATTERN_DARKGRID = "darkGrid"
FILL_PATTERN_DARKHORIZONTAL = "darkHorizontal"
FILL_PATTERN_DARKTRELLIS = "darkTrellis"
FILL_PATTERN_DARKUP = "darkUp"
FILL_PATTERN_DARKVERTICAL = "darkVertical"
FILL_PATTERN_GRAY0625 = "gray0625"
FILL_PATTERN_GRAY125 = "gray125"
FILL_PATTERN_LIGHTDOWN = "lightDown"
FILL_PATTERN_LIGHTGRAY = "lightGray"
FILL_PATTERN_LIGHTGRID = "lightGrid"
FILL_PATTERN_LIGHTHORIZONTAL = "lightHorizontal"
FILL_PATTERN_LIGHTTRELLIS = "lightTrellis"
FILL_PATTERN_LIGHTUP = "lightUp"
FILL_PATTERN_LIGHTVERTICAL = "lightVertical"
FILL_PATTERN_MEDIUMGRAY = "mediumGray"

fills = (
    FILL_SOLID,
    FILL_PATTERN_DARKDOWN,
    FILL_PATTERN_DARKGRAY,
    FILL_PATTERN_DARKGRID,
    FILL_PATTERN_DARKHORIZONTAL,
    FILL_PATTERN_DARKTRELLIS,
    FILL_PATTERN_DARKUP,
    FILL_PATTERN_DARKVERTICAL,
    FILL_PATTERN_GRAY0625,
    FILL_PATTERN_GRAY125,
    FILL_PATTERN_LIGHTDOWN,
    FILL_PATTERN_LIGHTGRAY,
    FILL_PATTERN_LIGHTGRID,
    FILL_PATTERN_LIGHTHORIZONTAL,
    FILL_PATTERN_LIGHTTRELLIS,
    FILL_PATTERN_LIGHTUP,
    FILL_PATTERN_LIGHTVERTICAL,
    FILL_PATTERN_MEDIUMGRAY,
)

def _assign_position(
    values: Union[List[Color], List[Stop], Tuple, List[Union[Stop, str]], List[str]]
) -> List[Stop]: ...

class Fill:
    @classmethod
    def from_tree(cls, el: _Element) -> Optional[Union[GradientFill, PatternFill]]: ...

class GradientFill:
    def __init__(
        self,
        type: str = ...,
        degree: Union[str, int] = ...,
        left: Union[str, int] = ...,
        right: Union[str, int] = ...,
        top: Union[str, int] = ...,
        bottom: Union[str, int] = ...,
        stop: Union[List[Stop], Tuple] = ...,
    ) -> None: ...
    def __iter__(self) -> Iterator[Tuple[str, str]]: ...
    def to_tree(
        self, tagname: None = ..., namespace: None = ..., idx: None = ...
    ) -> _Element: ...

class PatternFill:
    tagname = "patternFill"

    __elements__ = ("fgColor", "bgColor")

    patternType = NoneSet(values=fills)
    fill_type = Alias("patternType")
    fgColor = ColorDescriptor()
    start_color = Alias("fgColor")
    bgColor = ColorDescriptor()
    end_color = Alias("bgColor")
    def __init__(
        self,
        patternType: Optional[str] = ...,
        fgColor: Union[str, Color] = ...,
        bgColor: Union[str, Color] = ...,
        fill_type: Optional[str] = ...,
        start_color: Optional[Union[str, Color]] = ...,
        end_color: Optional[Union[str, Color]] = ...,
    ) -> None:
        if fill_type is not None:
            patternType = fill_type
        self.patternType = patternType
        if start_color is not None:
            fgColor = start_color
        self.fgColor = fgColor
        if end_color is not None:
            bgColor = end_color
        self.bgColor = bgColor
    @classmethod
    def _from_tree(cls, el: _Element) -> PatternFill: ...
    def to_tree(self, tagname: Optional[str] = ..., idx: None = ...) -> _Element: ...

class Stop:
    def __init__(
        self, color: Union[str, Color], position: Optional[Union[float, str, int]]
    ) -> None: ...

class StopList:
    def __set__(self, obj: GradientFill, values: Union[List[Stop], Tuple]) -> None: ...



from openpyxl.cell.cell import Cell
from openpyxl.worksheet._write_only import WriteOnlyWorksheet
from openpyxl.worksheet.worksheet import Worksheet
from typing import (
    Any,
    Iterator,
    List,
    Optional,
    Tuple,
    Union,
)

def create_temporary_file(suffix: str = ...) -> str: ...

class WorksheetWriter:
    def __init__(
        self, ws: Union[Worksheet, WriteOnlyWorksheet], out: None = ...
    ) -> None: ...
    def cleanup(self) -> None: ...
    def close(self) -> None: ...
    def get_stream(self) -> Iterator[None]: ...
    def read(self) -> bytes: ...
    def rows(
        self,
    ) -> Union[
        List[Union[Tuple[int, List[Any]], Tuple[int, List[Cell]]]],
        List[Tuple[int, List[Cell]]],
    ]: ...
    def write(self) -> None: ...
    def write_breaks(self) -> None: ...
    def write_cols(self) -> None: ...
    def write_dimensions(self) -> None: ...
    def write_drawings(self) -> None: ...
    def write_filter(self) -> None: ...
    def write_format(self) -> None: ...
    def write_formatting(self) -> None: ...
    def write_header(self) -> None: ...
    def write_hyperlinks(self) -> None: ...
    def write_legacy(self) -> None: ...
    def write_margins(self) -> None: ...
    def write_merged_cells(self) -> None: ...
    def write_page(self) -> None: ...
    def write_print(self) -> None: ...
    def write_properties(self) -> None: ...
    def write_protection(self) -> None: ...
    def write_rows(self) -> None: ...
    def write_scenarios(self) -> None: ...
    def write_sort(self) -> None: ...
    def write_tables(self) -> None: ...
    def write_tail(self) -> None: ...
    def write_top(self) -> None: ...
    def write_validations(self) -> None: ...
    def write_views(self) -> None: ...



from datetime import date
from lxml.etree import _Element
from openpyxl.cell.cell import Cell
from openpyxl.worksheet.worksheet import Worksheet
from typing import (
    Any,
    Dict,
    Iterator,
    List,
    Optional,
    Set,
    Tuple,
    Union,
)
from xml.etree.ElementTree import Element
from zipfile import ZipExtFile

def _cast_number(value: str) -> Union[float, int]: ...

class WorkSheetParser:
    def __init__(
        self,
        src: Optional[Union[ZipExtFile, str]],
        shared_strings: Optional[Union[Dict[int, str], List[str]]],
        data_only: Optional[bool] = ...,
        epoch: Optional[Union[date, float]] = ...,
        date_formats: Set[int] = ...,
    ) -> None: ...
    def parse(self) -> Iterator[Any]: ...
    def parse_cell(self, element: Union[_Element, Element]) -> Dict[str, Any]: ...
    def parse_col_breaks(self, element: _Element) -> None: ...
    def parse_column_dimensions(self, col: Union[_Element, Element]) -> None: ...
    def parse_custom_views(self, element: Element) -> None: ...
    def parse_dimensions(
        self,
    ) -> Optional[Union[Tuple[int, int, int, int], Tuple[None, int, None, int]]]: ...
    def parse_extensions(self, element: Union[_Element, Element]) -> None: ...
    def parse_formatting(self, element: Union[_Element, Element]) -> None: ...
    def parse_formula(self, element: Union[_Element, Element]) -> str: ...
    def parse_legacy(self, element: Union[_Element, Element]) -> None: ...
    def parse_row(self, row: Union[_Element, Element]) -> Any: ...
    def parse_row_breaks(self, element: Union[_Element, Element]) -> None: ...
    def parse_sheet_protection(self, element: Union[_Element, Element]) -> None: ...

class WorksheetReader:
    def __init__(
        self,
        ws: Worksheet,
        xml_source: Union[ZipExtFile, str],
        shared_strings: Optional[List[str]],
        data_only: Optional[bool],
    ) -> None: ...
    def bind_all(self) -> None: ...
    def bind_cells(self) -> None: ...
    def bind_col_dimensions(self) -> None: ...
    def bind_formatting(self) -> None: ...
    def bind_hyperlinks(self) -> None: ...
    def bind_merged_cells(self) -> None: ...
    def bind_properties(self) -> None: ...
    def bind_row_dimensions(self) -> None: ...
    def bind_tables(self) -> None: ...
    def normalize_merged_cell_link(self, coord: str) -> Optional[Cell]: ...



from openpyxl.worksheet.worksheet import Worksheet
from typing import (
    Optional,
    Union,
)

class WorksheetCopy:
    def __init__(
        self,
        source_worksheet: Union[str, Worksheet],
        target_worksheet: Optional[Worksheet],
    ) -> None: ...
    def _copy_cells(self) -> None: ...
    def _copy_dimensions(self) -> None: ...
    def _verify_resources(self) -> None: ...
    def copy_worksheet(self) -> None: ...



# Core Library modules
import datetime
from decimal import Decimal
from typing import Any, Iterator, Optional, Tuple, Union

# Third party modules
from openpyxl.cell.cell import Cell, MergedCell
from openpyxl.cell.read_only import ReadOnlyCell
from openpyxl.chart.bar_chart import BarChart
from openpyxl.descriptors import String
from openpyxl.drawing.image import Image
from openpyxl.workbook.child import _WorkbookChild
from openpyxl.workbook.workbook import Workbook
from openpyxl.worksheet.cell_range import CellRange
from openpyxl.worksheet.dimensions import ColumnDimension, RowDimension
from openpyxl.worksheet.merge import MergedCellRange
from openpyxl.worksheet.table import Table, TableList
from openpyxl.worksheet.tests.test_dimensions import DummyWorkbook
from openpyxl.worksheet.tests.test_worksheet import DummyWorkbook
from openpyxl.worksheet.views import SheetView

def _gutter(idx: int, offset: int, max_val: int) -> range: ...

class Worksheet(_WorkbookChild):
    def __delitem__(self, key: str) -> None: ...
    def __getitem__(self, key: Union[str, slice, int]) -> Any: ...
    def __init__(
        self,
        parent: Union[DummyWorkbook, DummyWorkbook, Workbook],
        title: Optional[Union[str, int]] = ...,
    ) -> None:
        _WorkbookChild.__init__(self, parent, title)
        self._setup()
    def _setup(self):
        self.row_dimensions = DimensionHolder(
            worksheet=self, default_factory=self._add_row
        )
        self.column_dimensions = DimensionHolder(
            worksheet=self, default_factory=self._add_column
        )
        self.row_breaks = RowBreak()
        self.col_breaks = ColBreak()
        self._cells = {}
        self._charts = []
        self._images = []
        self._rels = RelationshipList()
        self._drawing = None
        self._comments = []
        self.merged_cells = MultiCellRange()
        self._tables = TableList()
        self._pivots = []
        self.data_validations = DataValidationList()
        self._hyperlinks = []
        self.sheet_state = "visible"
        self.page_setup = PrintPageSetup(worksheet=self)
        self.print_options = PrintOptions()
        self._print_rows = None
        self._print_cols = None
        self._print_area = None
        self.page_margins = PageMargins()
        self.views = SheetViewList()
        self.protection = SheetProtection()

        self._current_row = 0
        self.auto_filter = AutoFilter()
        self.paper_size = None
        self.formula_attributes = {}
        self.orientation = None
        self.conditional_formatting = ConditionalFormattingList()
        self.legacy_drawing = None
        self.sheet_properties = WorksheetProperties()
        self.sheet_format = SheetFormatProperties()
        self.scenarios = ScenarioList()
    def __iter__(self) -> Iterator[Any]: ...
    def __setitem__(self, key: str, value: Union[str, int]) -> None: ...
    def _add_cell(self, cell: Cell) -> None: ...
    def _add_column(self) -> ColumnDimension: ...
    def _add_row(self) -> RowDimension: ...
    def _cells_by_col(
        self,
        min_col: int,
        min_row: int,
        max_col: int,
        max_row: int,
        values_only: bool = ...,
    ) -> Iterator[Tuple[Cell, ...]]: ...
    def _cells_by_row(
        self,
        min_col: int,
        min_row: int,
        max_col: int,
        max_row: int,
        values_only: bool = ...,
    ) -> Iterator[Any]: ...
    def _clean_merge_range(self, mcr: MergedCellRange) -> None: ...
    def _get_cell(self, row: int, column: int) -> Union[MergedCell, Cell]: ...
    def _invalid_row(self, iterable: str): ...
    def _move_cell(
        self,
        row: int,
        column: int,
        row_offset: int,
        col_offset: int,
        translate: bool = ...,
    ) -> None: ...
    def _move_cells(
        self,
        min_row: Optional[int] = ...,
        min_col: Optional[int] = ...,
        offset: int = ...,
        row_or_col: str = ...,
    ) -> None: ...
    @property
    def active_cell(self) -> str: ...
    def add_chart(self, chart: BarChart, anchor: Optional[str] = ...) -> None: ...
    def add_image(self, img: Image, anchor: Optional[str] = ...) -> None: ...
    def add_table(self, table: Table) -> None: ...
    def append(self, iterable: Any) -> None: ...
    def calculate_dimension(self) -> str: ...
    def cell(
        self,
        row: int,
        column: int,
        value: Optional[
            Union[
                int,
                float,
                Decimal,
                str,
                bytes,
                datetime.datetime,
                datetime.date,
                datetime.time,
                datetime.timedelta,
            ]
        ] = ...,
    ) -> Union[MergedCell, ReadOnlyCell, Cell]: ...
    @property
    def columns(self) -> Iterator[Any]: ...
    def delete_cols(self, idx: int, amount: int = ...) -> None: ...
    def delete_rows(self, idx: int, amount: int = ...) -> None: ...
    def insert_cols(self, idx: int, amount: int = ...) -> None: ...
    def insert_rows(self, idx: int, amount: int = ...) -> None: ...
    def iter_cols(
        self,
        min_col: Optional[int] = ...,
        max_col: Optional[int] = ...,
        min_row: Optional[int] = ...,
        max_row: Optional[int] = ...,
        values_only: bool = ...,
    ) -> Iterator[Any]: ...
    def iter_rows(
        self,
        min_row: Optional[int] = ...,
        max_row: Optional[int] = ...,
        min_col: Optional[int] = ...,
        max_col: Optional[int] = ...,
        values_only: bool = ...,
    ) -> Iterator[Any]: ...
    @property
    def max_column(self) -> int: ...
    @property
    def max_row(self) -> int: ...
    def merge_cells(
        self,
        range_string: Optional[str] = ...,
        start_row: Optional[int] = ...,
        start_column: Optional[int] = ...,
        end_row: Optional[int] = ...,
        end_column: Optional[int] = ...,
    ) -> None: ...
    @property
    def min_column(self) -> int: ...
    @property
    def min_row(self) -> int: ...
    def move_range(
        self,
        cell_range: Union[CellRange, str],
        rows: int = ...,
        cols: int = ...,
        translate: bool = ...,
    ) -> None: ...
    @property
    def print_titles(self) -> Optional[str]: ...
    @property
    def rows(self) -> Iterator[Any]: ...
    @property
    def selected_cell(self) -> str: ...
    @property
    def sheet_view(self) -> SheetView: ...
    @property
    def show_gridlines(self) -> None: ...
    @property
    def tables(self) -> TableList: ...
    def unmerge_cells(
        self,
        range_string: Optional[str] = ...,
        start_row: Optional[int] = ...,
        start_column: Optional[int] = ...,
        end_row: Optional[int] = ...,
        end_column: Optional[int] = ...,
    ) -> None: ...
    @property
    def values(
        self,
    ) -> Iterator[
        Union[
            Tuple[str, None, None, None, None, None, None],
            Tuple[None, None, None, None, None, None, None],
            Tuple[None, None, None, None, None, None, str],
        ]
    ]: ...
    @property
    def freeze_panes(self) -> Optional[String]:
        if self.sheet_view.pane is not None:
            return self.sheet_view.pane.topLeftCell
        return None
    @freeze_panes.setter
    def freeze_panes(self, topLeftCell=None) -> None: ...



from typing import (
    Any,
    List,
    Optional,
    Union,
)
from openpyxl.worksheet.header_footer import HeaderFooter

def avoid_duplicate_name(names: List[str], value: str) -> str: ...

class _WorkbookChild:

    __title = ""
    _id = None
    _path = "{0}"
    _parent = None
    _default_title = "Sheet"
    def __init__(
        self, parent: Optional[Any] = ..., title: Optional[Union[bytes, str, int]] = ...
    ) -> None:
        self._parent = parent
        self.title = title or self._default_title
        self.HeaderFooter = HeaderFooter()
    def __repr__(self) -> str: ...
    @property
    def encoding(self) -> str: ...
    @property
    def parent(self) -> Any: ...
    @property
    def path(self) -> str: ...



from .alignment import Alignment as Alignment
from .borders import Border as Border, Side as Side
from .colors import Color as Color
from .fills import (
    PatternFill as PatternFill,
    GradientFill as GradientFill,
    Fill as Fill,
)
from .fonts import Font as Font, DEFAULT_FONT as DEFAULT_FONT
from .numbers import (
    NumberFormatDescriptor as NumberFormatDescriptor,
    is_date_format as is_date_format,
    is_builtin as is_builtin,
)
from .protection import Protection as Protection
from .named_styles import NamedStyle as NamedStyle
